package de.intevation.gnv.action;

import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;

import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClient;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;

import de.intevation.gnv.artifactdatabase.objects.ArtifactDescription;
import de.intevation.gnv.artifactdatabase.objects.ArtifactObject;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;

import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;


/**
 * @author Ingo Weinzierl <ingo.weinzierl@intevation.de>
 *
 */
public class PreviousArtifactStepAction extends DescribeUIAction {


    public static final String URL_STATE_KEY = "target";

    private static Logger logger =
        Logger.getLogger(PreviousArtifactStepAction.class);


    public PreviousArtifactStepAction() {
        super();
    }


    @Override
    public ActionForward execute(
        ActionMapping       mapping,
        ActionForm          form,
        HttpServletRequest  request,
        HttpServletResponse response
    ) throws Exception {

        String id = request.getSession().getId();

        SessionModel session = SessionModelFactory.getInstance().getSessionModel(
            request);

        ArtifactDescription ad = session.getArtifactDescription();
        if (ad == null) {
            logger.warn("Session timed out.");
            request.setAttribute(
                CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID, 
                "SessionTimeout has occured");
            new FetchArtifactFactoriesAction().execute(
                mapping, form, request, response);

            return super.execute(mapping, form, request, response);
        }

        // TODO check if target is reachable

        ArtifactDatabaseClientFactory f =
            ArtifactDatabaseClientFactory.getInstance();
        ArtifactDatabaseClient client   =
            f.getArtifactDatabaseClient(getLocale(request));

        ArtifactObject artifactFactory = session.getSelectedArtifactFactory();
        ArtifactObject currentArtifact = session.getCurrentArtifact();

        String target = (String) request.getParameter(URL_STATE_KEY);
        logger.debug("Step back to previous state: " + target);

        try {
            client.doAdvance(
                artifactFactory,
                currentArtifact,
                target
            );
        }
        catch (Exception e) {
            logger.error(e, e);
            request.setAttribute(
                CommunicationKeys.REQUEST_EXCEPTION_MESSAGE_ID,
                e.getMessage());
        }

        return super.execute(mapping, form, request, response);
    }
}
// vim:set ts=4 sw=4 si et sta sts=4 fenc=utf-8 :
